import { world, system, ItemStack, Container } from '@minecraft/server';
import { ActionFormData, ModalFormData } from '@minecraft/server-ui';

let powerStoneCooldowns = new Map();
const POWER_STONE_COOLDOWN = 2 * 60 * 1000; // 3 minutes in milliseconds
let playerOriginalData = new Map(); // Track player's original data
let playerControlIntervals = new Map(); // Track control intervals
let playerControlledMobs = new Map(); // Track controlled mobs
let playerNoHitboxEntities = new Map(); // Track no hitbox entities
let mobControlStatus = new Map(); // Track mob control status
let playerTimeData = new Map(); // Store player data for time tracking

function showActionForm(player) {
    const actionForm = new ActionFormData().title("Infinity Gauntlet");

    if (playerOriginalData.has(player.name)) {
        actionForm.button("Exit Mob Control", "textures/items/door_iron");
    } else {
        actionForm
            .button("Space Stone", "textures/items/space_stone")
            .button("Time Stone", "textures/items/time_stone")
            .button("Power Stone", "textures/items/power_stone")
            .button("Mind Stone", "textures/items/mind_stone")
            .button("Soul Stone", "textures/items/soul_stone")
            .button("Reality Stone", "textures/items/reality_stone")
            .button("Help", "textures/items/infinity_gauntlet");

        if (player.hasTag("admin")) {
            actionForm.button("Snap", "textures/items/infinity_gauntlet");
        }
    }

    actionForm.show(player).then(response => {
        if (response.canceled) return;

        if (playerOriginalData.has(player.name)) {
            exitMobControl(player);
        } else {
            switch (response.selection) {
                case 0:
                    showTeleportForm(player);
                    break;
                case 1:
                    showTimeStoneMenu(player);
                    break;
                case 2:
                    showPowerStoneMenu(player);
                    break;
                case 3:
                    showMindStoneMenu(player);
                    break;
                case 4:
                    showSoulStoneMenu(player);
                    break;
                case 5:
                    showRealityStoneMenu(player);
                    break;
                case 6:
                    showHelpMenu(player);
                    break;
                case 7:
                    if (player.hasTag("admin")) {
                        snap(player);
                    }
                    break;
            }
        }
    });
}
let mobControlExitGracePeriod = new Map(); // Track players' grace period after exiting mob control
const GRACE_PERIOD_DURATION = 5 * 1000; // 5 seconds in milliseconds

function applyWitheringIfNotFullArmor() {
    const commands = [
        "/execute as @a[hasitem={item=regentstew:infinity_boots, location=slot.armor.feet, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:time_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_leggings, location=slot.armor.legs, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:time_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_chestplate, location=slot.armor.chest, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:time_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_helmet, location=slot.armor.head, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:time_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",

        "/execute as @a[hasitem={item=regentstew:infinity_boots, location=slot.armor.feet, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:space_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_leggings, location=slot.armor.legs, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:space_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_chestplate, location=slot.armor.chest, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:space_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_helmet, location=slot.armor.head, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:space_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",

        "/execute as @a[hasitem={item=regentstew:infinity_boots, location=slot.armor.feet, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:power_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_leggings, location=slot.armor.legs, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:power_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_chestplate, location=slot.armor.chest, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:power_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_helmet, location=slot.armor.head, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:power_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",

        "/execute as @a[hasitem={item=regentstew:infinity_boots, location=slot.armor.feet, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:mind_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_leggings, location=slot.armor.legs, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:mind_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_chestplate, location=slot.armor.chest, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:mind_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_helmet, location=slot.armor.head, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:mind_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",

        "/execute as @a[hasitem={item=regentstew:infinity_boots, location=slot.armor.feet, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:reality_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_leggings, location=slot.armor.legs, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:reality_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_chestplate, location=slot.armor.chest, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:reality_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_helmet, location=slot.armor.head, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:reality_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",

        "/execute as @a[hasitem={item=regentstew:infinity_boots, location=slot.armor.feet, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:soul_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_leggings, location=slot.armor.legs, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:soul_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_chestplate, location=slot.armor.chest, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:soul_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_helmet, location=slot.armor.head, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:soul_stone, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",

        "/execute as @a[hasitem={item=regentstew:infinity_boots, location=slot.armor.feet, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:infinity_gauntlet, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_leggings, location=slot.armor.legs, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:infinity_gauntlet, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_chestplate, location=slot.armor.chest, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:infinity_gauntlet, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true",
        "/execute as @a[hasitem={item=regentstew:infinity_helmet, location=slot.armor.head, quantity=0}] at @s run /execute as @s[hasitem={item=regentstew:infinity_gauntlet, location=slot.weapon.mainhand}, m=!c] at @s run /effect @s wither 5 9 true"
    ];

    commands.forEach(command => {
        world.getPlayers().forEach(player => {
            const now = Date.now();
            const gracePeriodEnd = mobControlExitGracePeriod.get(player.name);
            if (!gracePeriodEnd || now > gracePeriodEnd) {
                player.runCommand(command);
            }
        });
    });
}
system.runInterval(() => {
    applyWitheringIfNotFullArmor();
}, 1); // 1 tick = 0.05 second

function showTeleportForm(player) {
    const modalForm = new ModalFormData()
        .title("Space Stone")
        .textField("Enter X coordinate:", "~")
        .textField("Enter Y coordinate:", "~")
        .textField("Enter Z coordinate:", "~")
        .dropdown("Select Dimension:", ["Overworld", "Nether", "End"]);

    modalForm.show(player).then(response => {
        if (response.canceled) return;

        const x = response.formValues[0] === "" ? "~" : response.formValues[0];
        const y = response.formValues[1] === "" ? "~" : response.formValues[1];
        const z = response.formValues[2] === "" ? "~" : response.formValues[2];
        const dimension = response.formValues[3];

        if ((typeof x === 'number' && isNaN(x)) || (typeof y === 'number' && isNaN(y)) || (typeof z === 'number' && isNaN(z)) || y > 320 || y < -64) {
            player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Invalid coordinates. Please enter valid numbers and ensure the Y coordinate is between -64 and 320."}]}`);
            return;
        }

        let dimensionCommand;
        switch (dimension) {
            case 1: // Nether
                dimensionCommand = "nether";
                break;
            case 2: // End
                dimensionCommand = "the_end";
                break;
            case 0: // Overworld
            default:
                dimensionCommand = "overworld";
        }

        // Debugging message
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Teleporting to dimension: ${dimensionCommand}"}]}`);

        player.runCommandAsync(`execute in ${dimensionCommand} run tp ${player.name} ${x} ${y} ${z}`);
    });
}

function showTimeStoneMenu(player) {
    const timeStoneForm = new ActionFormData()
        .title("Time Stone")
        .button("Set Time", "textures/items/clock_item")
        .button("Manage Time Points", "textures/items/clock_item")
        .button("Freeze Entities", "textures/items/snowball");

    timeStoneForm.show(player).then(response => {
        if (response.canceled) return;

        switch (response.selection) {
            case 0: // Set Time
                showSetTimeForm(player);
                break;
            case 1: // Manage Time Points
                showTimePointsMenu(player);
                break;
            case 2: // Freeze Entities
                freezeEntitiesInRadius(player);
                break;
        }
    });
}

// Listen for item use
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const itemStack = event.itemStack;

    switch (itemStack.typeId) {
        case "regentstew:space_stone":
            showTeleportForm(player);
            break;
        case "regentstew:time_stone":
            showTimeStoneMenu(player);
            break;
        case "regentstew:power_stone":
            showPowerStoneMenu(player);
            break;
        case "regentstew:mind_stone":
            showMindStoneMenu(player);
            break;
        case "regentstew:soul_stone": // Added this case
            showSoulStoneMenu(player);
            break;
        case "regentstew:reality_stone": // Added this case
            showRealityStoneMenu(player);
            break;
        case "regentstew:help": // Added this case
            showHelpMenu(player);
            break;
        case "regentstew:exit_mob_control":
            if (player.hasTag("controlling_mob")) {
                exitMobControl(player);
            } else {
                player.runCommandAsync(`effect ${player.name} clear`); // Clear all effects if not controlling a mob
                player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"No mob control active. Cleared effects."}]}`);
            }
            break;
        default:
            break;
    }
});

function freezeEntitiesInRadius(player) {
    const radius = 30;
    const freezeDuration = 30; // Duration in seconds
    const dimensionId = player.dimension.id;
    const playerLocation = player.location;

    // Ensure scoreboard objective exists
    player.runCommandAsync('scoreboard objectives add FreezeTime dummy');

    // Apply freeze effects to players and entities within the radius
    const entities = world.getDimension(dimensionId).getEntities();

    entities.forEach(entity => {
        const distance = Math.sqrt(
            Math.pow(entity.location.x - playerLocation.x, 2) +
            Math.pow(entity.location.y - playerLocation.y, 2) +
            Math.pow(entity.location.z - playerLocation.z, 2)
        );

        if (distance <= radius) {
            if (entity.typeId === "minecraft:player") {
                if (entity.name !== player.name && !entity.hasTag("admin")) { // Exclude the player who initiated the freeze and players with the "admin" tag
                    // Apply movement and camera freezing to players
                    entity.runCommandAsync(`/inputpermission set ${entity.name} movement disabled`);
                    entity.runCommandAsync(`/inputpermission set ${entity.name} camera disabled`);

                    // Set freeze time for players
                    entity.runCommandAsync(`scoreboard players set ${entity.name} FreezeTime ${freezeDuration}`);
                }
            } else {
                // Apply Slowness 255 to other entities
                entity.runCommandAsync(`effect @s slowness ${freezeDuration} 255 true`);
            }
        }
    });

    player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Stopped time for all entities nearby."}]}`);
}

// Single global interval to handle timing
let intervalRunning = false;
if (!intervalRunning) {
    intervalRunning = true;
    system.runInterval(() => {
        // Decrement the freeze time for players
        world.getDimension('overworld').runCommandAsync('scoreboard players add @a[scores={FreezeTime=1..}] FreezeTime -1');

        // Check and unfreeze players with FreezeTime score of 0
        world.getDimension('overworld').runCommandAsync('/execute as @a[scores={FreezeTime=0}] run /inputpermission set @s movement enabled');
        world.getDimension('overworld').runCommandAsync('/execute as @a[scores={FreezeTime=0}] run /inputpermission set @s camera enabled');
        world.getDimension('overworld').runCommandAsync('/execute as @a[scores={FreezeTime=0}] run /scoreboard players reset @s FreezeTime');

        // Repeat for other dimensions if needed
        // world.getDimension('nether').runCommandAsync(...);
        // world.getDimension('the_end').runCommandAsync(...);
    }, 20); // 20 ticks = 1 second
}



function showTimePointsMenu(player) {
    const timePointsForm = new ActionFormData()
        .title("Time Points")
        .button("Save Point in Time", "textures/items/clock_item")
        .button("Go to Last Point in Time", "textures/items/clock_item");

    timePointsForm.show(player).then(response => {
        if (response.canceled) return;

        switch (response.selection) {
            case 0: // Save Point in Time
                savePlayerPoint(player);
                player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Point in time saved."}]}`);
                break;
            case 1: // Go to Last Point in Time
                goToLastPointInTime(player);
                break;
        }
    });
}

function savePlayerPoint(player) {
    const location = player.location;
    const timestamp = Date.now();

    // Save the player's location and timestamp
    playerTimeData.set(player.name, {
        lastLocation: location,
        lastSaved: timestamp
    });
}


function goToLastPointInTime(player) {
    const playerData = playerTimeData.get(player.name);

    if (playerData) {
        const { lastLocation } = playerData;
        player.teleport(lastLocation);
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Returned to your saved location."}]}`);
    } else {
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"No saved point in time found. Use 'Save Point in Time' first."}]}`);
    }
}

function showSetTimeForm(player) {
    const timeForm = new ModalFormData().title("Time Stone").textField("Enter time value:", "1000");

    timeForm.show(player).then(response => {
        if (response.canceled) return;

        const timeValue = parseInt(response.formValues[0]);

        if (isNaN(timeValue) || timeValue < 0 || timeValue > 24000) {
            player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Invalid time value. Please enter a number between 0 and 24000."}]}`);
            return;
        }

        player.runCommandAsync(`time set ${timeValue}`);
    });
}

function showPowerStoneMenu(player) {
    const now = Date.now();
    const lastUsed = powerStoneCooldowns.get(player.name);
    const isOnCooldown = lastUsed && now - lastUsed < POWER_STONE_COOLDOWN;

    const powerStoneForm = new ActionFormData().title("Power Stone");

    if (isOnCooldown) {
        const timeLeft = Math.ceil((POWER_STONE_COOLDOWN - (now - lastUsed)) / 1000);
        powerStoneForm.button(`Power Stone (Cooldown: ${timeLeft} seconds)`, "textures/blocks/barrier");
    } else {
        powerStoneForm.button("Activate Power Stone", "textures/items/nether_star");
    }

    powerStoneForm.show(player).then(response => {
        if (response.canceled) return;

        if (!isOnCooldown && response.selection === 0) {
            activatePowerStone(player);
        }
    });
}

    function activatePowerStone(player) {
        const now = Date.now();
        const radius = 30; // Radius within which the wither effect will be applied
    
        // Apply effects to the player using the Power Stone
        player.runCommandAsync(`effect ${player.name} strength 60 2 true`);
        player.runCommandAsync(`effect ${player.name} speed 60 1 true`);
        player.runCommandAsync(`effect ${player.name} resistance 60 1 true`);
        player.runCommandAsync(`effect ${player.name} health_boost 60 10 true`);
        player.runCommandAsync(`effect ${player.name} regeneration 5 255 true`);
        powerStoneCooldowns.set(player.name, now);
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Power Stone activated!"}]}`);
    
        // Get the player's current dimension and location
        const dimension = player.dimension;
        const playerLocation = player.location;
    
        // Apply Wither IV effect to all entities except the player within the radius
        dimension.getEntities().forEach(entity => {
            if (entity.id !== player.id) {
                const distance = Math.sqrt(
                    Math.pow(entity.location.x - playerLocation.x, 2) +
                    Math.pow(entity.location.y - playerLocation.y, 2) +
                    Math.pow(entity.location.z - playerLocation.z, 2)
                );
    
                if (distance <= radius) {
                    entity.runCommandAsync(`effect @s wither 15 3 true`);
                }
            }
        });
    }    

    function showMindStoneMenu(player) {
        const mindStoneForm = new ActionFormData()
            .title("Mind Stone")
            .button("Control Mob", "textures/items/compass_item")
            .button("Blind Mobs", "textures/items/spider_eye_fermented");
    
        mindStoneForm.show(player).then(response => {
            if (response.canceled) return;
    
            switch (response.selection) {
                case 0:
                    showControlMobMenu(player);
                    break;
                case 1:
                    showBlindMobsMenu(player);
                    break;
            }
        });
    }

    function showControlMobMenu(player) {
        const playerPosition = player.location;
        const isControllingMob = playerOriginalData.has(player.name);
    
        // Get all nearby mobs
        const mobs = world.getDimension(player.dimension.id).getEntities().filter(entity => {
            const entityType = entity.typeId;
            return !["minecraft:player", "minecraft:item", "minecraft:tnt", "minecraft:ender_crystal", "minecraft:xp_orb"].includes(entityType)
                && Math.sqrt(
                    Math.pow(entity.location.x - playerPosition.x, 2) +
                    Math.pow(entity.location.y - playerPosition.y, 2) +
                    Math.pow(entity.location.z - playerPosition.z, 2)
                ) <= 20;
        }).map(entity => ({
            entity,
            type: entity.typeId,
            distance: Math.sqrt(
                Math.pow(entity.location.x - playerPosition.x, 2) +
                Math.pow(entity.location.y - playerPosition.y, 2) +
                Math.pow(entity.location.z - playerPosition.z, 2)
            ).toFixed(2),
            location: entity.location
        }));
    
        mobs.sort((a, b) => a.distance - b.distance);
    
        const controlMobForm = new ActionFormData().title("Control Mob");
    
        // Add "Exit Mob Control" button only if the player is controlling a mob
        if (isControllingMob) {
            controlMobForm.button("Exit Mob Control", "textures/items/door_iron");
        }
    
        // Add buttons for nearby mobs or a no mobs message
        if (mobs.length === 0) {
            controlMobForm.button("No mobs found within 20 blocks", "textures/blocks/barrier");
        } else {
            // Only add mob buttons if the player is not controlling a mob
            if (!isControllingMob) {
                mobs.forEach((mob, index) => {
                    const displayName = mob.type.split(':')[1]; // Get the entity type name
                    controlMobForm.button(`${capitalizeFirstLetter(displayName)} (${mob.distance} blocks)`, "textures/items/compass_item");
                });
            }
        }
    
        controlMobForm.show(player).then(response => {
            if (response.canceled) return;
    
            if (isControllingMob && response.selection === 0) {
                exitMobControl(player);
            } else if (!isControllingMob && response.selection >= 0 && response.selection < mobs.length) {
                const selectedMob = mobs[response.selection];
                if (selectedMob) {
                    controlMob(player, selectedMob.entity);
                }
            }
        });
    }

    let blindMobsCooldowns = new Map();
    const BLIND_MOBS_COOLDOWN = 1 * 60 * 1000; // 1 minute in milliseconds
    
    function showBlindMobsMenu(player) {
        const now = Date.now();
        const lastUsed = blindMobsCooldowns.get(player.name);
        const isOnCooldown = lastUsed && now - lastUsed < BLIND_MOBS_COOLDOWN;
    
        const blindMobsForm = new ActionFormData().title("Blind Mobs");
    
        if (isOnCooldown) {
            const timeLeft = Math.ceil((BLIND_MOBS_COOLDOWN - (now - lastUsed)) / 1000);
            blindMobsForm.button(`Blind Mobs (Cooldown: ${timeLeft} seconds)`, "textures/blocks/barrier");
        } else {
            blindMobsForm.button("Activate Blind Mobs", "textures/items/spider_eye_fermented");
        }
    
        blindMobsForm.show(player).then(response => {
            if (response.canceled) return;
    
            if (!isOnCooldown && response.selection === 0) {
                activateBlindMobs(player);
            }
        });
    }
    
    function activateBlindMobs(player) {
        const now = Date.now();
        const radius = 10; // Radius within which the blindness effect will be applied
        const duration = 30; // Duration of blindness effect in seconds
    
        // Get the player's current dimension and location
        const dimension = player.dimension;
        const playerLocation = player.location;
    
        // Apply Blindness 255 effect to all mobs except the player within the radius
        dimension.getEntities().forEach(entity => {
            if (entity.id !== player.id) {
                const distance = Math.sqrt(
                    Math.pow(entity.location.x - playerLocation.x, 2) +
                    Math.pow(entity.location.y - playerLocation.y, 2) +
                    Math.pow(entity.location.z - playerLocation.z, 2)
                );
    
                if (distance <= radius) {
                    entity.runCommandAsync(`effect @s blindness ${duration} 255 true`);
                }
            }
        });
    
        blindMobsCooldowns.set(player.name, now);
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Blinded all mobs nearby."}]}`);
    }

function controlMob(player, mobEntity) {
    // Check if the mob is already controlled
    const controllingPlayer = mobControlStatus.get(mobEntity.id);
    if (controllingPlayer) {
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"This mob is already being controlled by ${controllingPlayer}. You cannot control it."}]}`);
        return;
    }

    // Ensure the player isn't already controlling a mob
    if (playerControlledMobs.has(player.name)) {
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"You are already controlling a mob. Exit the current control to control another mob."}]}`);
        return;
    }

    // Store the player's original position and dimension
    playerOriginalData.set(player.name, { location: player.location, dimension: player.dimension.id });

    // Tag the mob with the player's username
    const mobTag = `controlled_by_${player.name}`;
    mobEntity.addTag(mobTag);
    playerControlledMobs.set(player.name, mobEntity);
    mobControlStatus.set(mobEntity.id, player.name); // Mark mob as controlled by this player

    // Tag the player as controlling a mob
    player.addTag("controlling_mob");

    // Make the player invisible and give resistance and weakness 255
    player.runCommandAsync(`effect ${player.name} invisibility 99999 1 true`);
    player.runCommandAsync(`effect ${player.name} resistance 99999 255 true`);
    player.runCommandAsync(`effect ${player.name} weakness 99999 255 true`);

    // Teleport the player to the mob's location
    player.teleport(mobEntity.location); // Move the player to the mob's location

    // Give the player the exit mob control item
    player.runCommandAsync(`give ${player.name} regentstew:exit_mob_control 1`);

    // Start controlling the mob
    const intervalId = system.runInterval(() => {
        const mob = playerControlledMobs.get(player.name);
        if (mob) {
            // Make the mob face 3 blocks ahead of the player
            const playerLookVector = player.getViewDirection();
            const lookX = player.location.x + playerLookVector.x * 3;
            const lookY = player.location.y + playerLookVector.y * 3;
            const lookZ = player.location.z + playerLookVector.z * 3;
            mob.runCommandAsync(`tp @e[type=${mob.typeId},tag=${mobTag}] ${player.location.x} ${player.location.y} ${player.location.z} facing ${lookX} ${lookY} ${lookZ}`);
        } else {
            exitMobControl(player); // Exit control if the mob no longer exists (e.g., died)
        }
    }, 1);

    playerControlIntervals.set(player.name, intervalId);

    player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"You are now using the mind stone."}]}`);
}
function exitMobControl(player) {
    const originalData = playerOriginalData.get(player.name);
    if (originalData) {
        player.teleport(originalData.location);
        player.runCommandAsync(`effect ${player.name} clear`); // Clear all effects
        playerOriginalData.delete(player.name);

        const intervalId = playerControlIntervals.get(player.name);
        if (intervalId) {
            system.clearRun(intervalId);
            playerControlIntervals.delete(player.name);
        }

        const mobEntity = playerControlledMobs.get(player.name);
        if (mobEntity) {
            // Remove the tag from the mob
            const mobTag = `controlled_by_${player.name}`;
            mobEntity.removeTag(mobTag);
            playerControlledMobs.delete(player.name);
            mobControlStatus.delete(mobEntity.id); // Mark mob as no longer controlled
        }

        const dummyEntity = playerNoHitboxEntities.get(player.name);
        if (dummyEntity) {
            dummyEntity.kill(); // Remove the dummy entity
            playerNoHitboxEntities.delete(player.name);
        }

        // Remove the controlling_mob tag from the player
        player.removeTag("controlling_mob");

        // Remove the exit mob control item from the player's inventory
        player.runCommandAsync(`clear ${player.name} regentstew:exit_mob_control`);

        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"You have turned off the mind stone."}]}`);
    }
}
function capitalizeFirstLetter(string) {
    return string.charAt(0).toUpperCase() + string.slice(1);
}

// Listen for item use
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const itemStack = event.itemStack;

    if (itemStack.typeId === "regentstew:infinity_gauntlet") {
        showActionForm(player);
    }
});

// Listen for mob deaths to exit control automatically
world.afterEvents.entityDie.subscribe(event => {
    const entity = event.deadEntity;
    const mobTag = entity.getTags().find(tag => tag.startsWith("controlled_by_"));

    if (mobTag) {
        const playerName = mobTag.split("_")[2];
        const player = world.getPlayers().find(p => p.name === playerName);
        if (player) {
            exitMobControl(player);
        }
    }

    // Ensure mob is no longer marked as controlled
    mobControlStatus.delete(entity.id);
});
function showSoulStoneMenu(player) {
    const soulStoneForm = new ActionFormData()
        .title("Soul Stone")
        .button("Damage", "textures/items/blaze_powder")
        .button("Collect Souls", "textures/blocks/soul_sand")
        .button("Destroy", "textures/blocks/tnt_side")
        .button("Summon Allies", "textures/items/totem"); // New option for Summon Allies

    soulStoneForm.show(player).then(response => {
        if (response.canceled) return;

        switch (response.selection) {
            case 0:
                showDamageMenu(player);
                break;
            case 1:
                showCollectSoulsMenu(player);
                break;
            case 2:
                showDestroyMenu(player);
                break;
            case 3:
                showSummonMenu(player); // New function to show Summon menu
                break;
        }
    });
}
function showDestroyMenu(player) {
    const destroyForm = new ModalFormData()
        .title("Soul Stone - Destroy")
        .textField("Enter the number of souls to use (1-10):", "1");

    destroyForm.show(player).then(response => {
        if (response.canceled) return;

        const soulsToUse = parseInt(response.formValues[0]);

        if (isNaN(soulsToUse) || soulsToUse < 1 || soulsToUse > 10) {
            player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"Invalid number of souls. Please enter a number between 1 and 10."}]}`);
            return;
        }

        checkAndApplyDestroy(player, soulsToUse);
    }).catch(error => {
        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"An error occurred: ${error.message}"}]}`);
    });
}

// Function to check and apply the Destroy effect
function checkAndApplyDestroy(player, soulsToUse) {
    try {
        const soulCount = getPlayerSoulCount(player);

        if (soulCount < soulsToUse) {
            player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"You do not have enough souls."}]}`);
            return;
        }

        removePlayerSouls(player, soulsToUse);

        const radius = soulsToUse; // The radius of the destruction area
        const playerLocation = player.location;
        const dimension = player.dimension;
        const unbreakableBlocks = [
            "minecraft:bedrock", "minecraft:barrier", "minecraft:command_block", "minecraft:structure_block", 
            "minecraft:jigsaw", "minecraft:deny", "minecraft:allow",
            "minecraft:diamond_ore", "minecraft:deepslate_diamond_ore",
            "minecraft:emerald_ore", "minecraft:deepslate_emerald_ore",
            "minecraft:gold_ore", "minecraft:deepslate_gold_ore",
            "minecraft:iron_ore", "minecraft:deepslate_iron_ore",
            "minecraft:coal_ore", "minecraft:deepslate_coal_ore",
            "minecraft:nether_gold_ore", "minecraft:nether_quartz_ore",
            "minecraft:lapis_ore", "minecraft:deepslate_lapis_ore",
            "minecraft:redstone_ore", "minecraft:deepslate_redstone_ore",
            "minecraft:copper_ore", "minecraft:deepslate_copper_ore"
        ];

        let commands = [];
        for (let x = -radius; x <= radius; x++) {
            for (let y = -radius; y <= radius; y++) {
                for (let z = -radius; z <= radius; z++) {
                    const distance = Math.sqrt(x * x + y * y + z * z);
                    if (distance <= radius) {
                        const blockLocation = { x: playerLocation.x + x, y: playerLocation.y + y, z: playerLocation.z + z };
                        const block = dimension.getBlock(blockLocation);

                        if (block && !unbreakableBlocks.includes(block.typeId)) {
                            commands.push({ x: blockLocation.x, y: blockLocation.y, z: blockLocation.z });
                        }
                    }
                }
            }
        }

        // Process the commands in optimized batches
        processBlockDestruction(commands, dimension, player);

        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"Blocks destroyed using ${soulsToUse} souls."}]}`);
    } catch (error) {
        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"An error occurred."}]}`);
    }
}

// Function to process block destruction in optimized batches
function processBlockDestruction(blockLocations, dimension, player) {
    const batchSize = 100; // Adjust batch size to balance performance and memory usage
    let index = 0;

    function destroyNextBatch() {
        if (index >= blockLocations.length) {
            return;
        }

        let batch = blockLocations.slice(index, index + batchSize);
        batch.forEach(blockLocation => {
            dimension.runCommandAsync(`setblock ${blockLocation.x} ${blockLocation.y} ${blockLocation.z} air destroy`);
        });

        index += batchSize;
        system.runTimeout(destroyNextBatch, 1); // Small delay to avoid overloading the server
    }

    destroyNextBatch();
}
function showDamageMenu(player) {
    const damageForm = new ModalFormData()
        .title("Soul Stone - Damage")
        .textField("Enter the number of souls to use (1-50):", "1");

    damageForm.show(player).then(response => {
        if (response.canceled) return;

        const soulsToUse = parseInt(response.formValues[0]);

        if (isNaN(soulsToUse) || soulsToUse < 1 || soulsToUse > 50) {
            player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"Invalid number of souls. Please enter a number between 1 and 50."}]}`);
            return;
        }

        checkAndApplyDamage(player, soulsToUse);
    }).catch(error => {
        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"An error occurred: ${error.message}"}]}`);
    });
}

function checkAndApplyDamage(player, soulsToUse) {
    try {
        const soulCount = getPlayerSoulCount(player);

        if (soulCount < soulsToUse) {
            player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"You do not have enough souls."}]}`);
            return;
        }

        removePlayerSouls(player, soulsToUse);

        const radius = 10; // 10 blocks radius
        const damage = soulsToUse; // 1 heart (2 HP) per soul used
        const playerLocation = player.location;
        const dimension = player.dimension;

        // Apply damage to all entities within the radius
        dimension.getEntities().forEach(entity => {
            if (entity.id !== player.id) {
                const distance = Math.sqrt(
                    Math.pow(entity.location.x - playerLocation.x, 2) +
                    Math.pow(entity.location.y - playerLocation.y, 2) +
                    Math.pow(entity.location.z - playerLocation.z, 2)
                );

                if (distance <= radius) {
                    // Directly apply damage
                    entity.runCommandAsync(`damage @s ${damage}`);
                }
            }
        });

        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"Souls Released"}]}`);
    } catch (error) {
        player.runCommandAsync(`tellraw @s {"rawtext":[{"text":"An error occurred."}]}`);
    }
}
function getPlayerSoulCount(player) {
    let soulCount = 0;
    const inventory = player.getComponent('minecraft:inventory').container;

    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getItem(i);
        if (item && item.typeId === "regentstew:soul") {
            soulCount += item.amount;
        }
    }

    return soulCount;
}

// Function to remove souls from player's inventory
function removePlayerSouls(player, soulsToRemove) {
    const inventory = player.getComponent('minecraft:inventory').container;
    let soulsLeftToRemove = soulsToRemove;

    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getItem(i);
        if (item && item.typeId === "regentstew:soul") {
            if (item.amount > soulsLeftToRemove) {
                item.amount -= soulsLeftToRemove;
                inventory.setItem(i, item);
                break;
            } else {
                soulsLeftToRemove -= item.amount;
                inventory.setItem(i, null);
            }
        }
    }
}
function showCollectSoulsMenu(player) {
    const radius = 5; // 5 blocks radius
    const playerLocation = player.location;
    const dimension = player.dimension;

    // Define a set of entity types to exclude from soul collection
    const excludedEntityTypes = new Set([
        "minecraft:item",
        "minecraft:lightning_bolt",
        "minecraft:potion",
        // Add other entity types that should be excluded here
    ]);

    // Get nearby entities within the radius and filter out excluded entities
    const nearbyEntities = dimension.getEntities().filter(entity => {
        const distance = Math.sqrt(
            Math.pow(entity.location.x - playerLocation.x, 2) +
            Math.pow(entity.location.y - playerLocation.y, 2) +
            Math.pow(entity.location.z - playerLocation.z, 2)
        );
        return distance <= radius && entity.id !== player.id && !entity.hasTag("soul_collected_once") && !excludedEntityTypes.has(entity.typeId);
    });

    if (nearbyEntities.length === 0) {
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"No suitable entities found within 5 blocks."}]}`);
        return;
    }

    const entitySelectionForm = new ActionFormData().title("Select an Entity to Collect Souls");

    nearbyEntities.forEach(entity => {
        const entityName = entity.typeId.split(":")[1];
        const distance = Math.sqrt(
            Math.pow(entity.location.x - player.location.x, 2) +
            Math.pow(entity.location.y - player.location.y, 2) +
            Math.pow(entity.location.z - player.location.z, 2)
        ).toFixed(2);
        entitySelectionForm.button(`${entityName} (${distance} blocks away)`);
    });

    entitySelectionForm.show(player).then(response => {
        if (response.canceled) return;

        const selectedEntity = nearbyEntities[response.selection];
        if (selectedEntity) {
            collectSoulsFromMob(player, selectedEntity);
        }
    });
}
function collectSoulsFromMob(player, mob) {
    const chance = Math.random() * 100;
    let souls = 1; // 100% chance for 1 soul

    if (chance < 2) {
        souls += 13; // Total 14
    } else if (chance < 12) {
        souls += 5; // Total 6
    } else if (chance < 37) {
        souls += 3; // Total 4
    } else if (chance < 87) {
        souls += 1; // Total 2
    }

    // Apply slowness 255 for 30 seconds to the mob and add a different tag
    mob.addTag("soul_collected_once");

    // Give the collected souls to the player
    player.runCommandAsync(`/give ${player.name} regentstew:soul ${souls}`);

    // Apply slowness to the player only when souls are taken from them
    player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Collected ${souls} souls from the mob."}]}`);
}

function applyPermanentSlowness() {
    world.getDimension('overworld').runCommandAsync('effect @e[tag=soul_collected] slowness 1 255 true');
}

// Run the function every second (20 ticks)
system.runInterval(() => {
    applyPermanentSlowness();
}, 20);
const SUMMON_DURATION = 3 * 60 * 1000; // 3 minutes in milliseconds
function getMobTypeFromSelection(selection) {
    switch (selection) {
        case 0:
            return "minecraft:wolf";
        case 1:
            return "minecraft:iron_golem";
        case 2:
            return "minecraft:bee";
        default:
            return null;
    }
}

function showSummonMenu(player) {
    const summonForm = new ModalFormData()
        .title("Soul Stone - Summon Allies")
        .textField("Enter the number of souls to use (1-20):", "1");

    summonForm.show(player).then(response => {
        if (response.canceled) return;

        const soulsToUse = parseInt(response.formValues[0]);

        if (isNaN(soulsToUse) || soulsToUse < 1 || soulsToUse > 20) {
            player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Invalid number of souls. Please enter a number between 1 and 20."}]}`);
            return;
        }

        summonMobs(player, soulsToUse);
    });
}

function summonMobs(player, soulsToUse) {
    const soulCount = getPlayerSoulCount(player);

    if (soulCount < soulsToUse) {
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"You do not have enough souls."}]}`);
        return;
    }

    removePlayerSouls(player, soulsToUse);

    const playerLocation = player.location;
    const dimension = player.dimension;

    for (let i = 0; i < soulsToUse; i++) {
        dimension.runCommandAsync(`summon regentstew:ally_zombie ${playerLocation.x} ${playerLocation.y} ${playerLocation.z}`);
    }

    // Get all summoned mobs and set them to vanish after the duration
    const summonedMobs = dimension.getEntities().filter(entity => entity.typeId === "regentstew:ally_zombie" && entity.location.equals(playerLocation));
    summonedMobs.forEach(mob => {
        system.runTimeout(() => {
            mob.kill();
        }, SUMMON_DURATION);
    });

    player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Summoned ${soulsToUse} ally zombie(s) using ${soulsToUse} souls."}]}`);
}
function showRealityStoneMenu(player) {
    const realityStoneForm = new ActionFormData()
        .title("Reality Stone")
        .button("Locate Mobs", "textures/items/compass_item")
        .button("Set Camera", "textures/items/ender_eye")

    realityStoneForm.show(player).then(response => {
        if (response.canceled) return;

        switch (response.selection) {
            case 0:
                showLocateMobsForm(player);
                break;
            case 1:
                showSetCameraForm(player);
                break;
            case 2:
                showLocateStructureForm(player); // New case for Locate Structure
                break;
        }
    });
}

// Locate Mobs form
function showLocateMobsForm(player) {
    const locateMobsForm = new ModalFormData()
        .title("Locate Mobs")
        .textField("Enter the mob/player name:", "mob_name");

    locateMobsForm.show(player).then(response => {
        if (response.canceled) return;

        const mobName = response.formValues[0].toLowerCase();
        locateMobs(player, mobName);
    });
}

// Locate Mobs function
function locateMobs(player, mobName) {
    const radius = 1000; // Adjust the radius as needed
    const dimension = player.dimension;
    const playerLocation = player.location;

    const mobs = dimension.getEntities().filter(entity => {
        const entityType = entity.typeId.split(":")[1];
        const entityName = entity.name || entityType;
        const distance = Math.sqrt(
            Math.pow(entity.location.x - playerLocation.x, 2) +
            Math.pow(entity.location.y - playerLocation.y, 2) +
            Math.pow(entity.location.z - playerLocation.z, 2)
        );
        return (entityName.toLowerCase().includes(mobName) || entityType.toLowerCase().includes(mobName)) && distance <= radius;
    }).map(entity => ({
        entity,
        type: entity.typeId,
        name: entity.name || entity.typeId.split(":")[1],
        distance: Math.sqrt(
            Math.pow(entity.location.x - playerLocation.x, 2) +
            Math.pow(entity.location.y - playerLocation.y, 2) +
            Math.pow(entity.location.z - playerLocation.z, 2)
        ).toFixed(2),
        location: entity.location
    }));

    mobs.sort((a, b) => a.distance - b.distance);
    const closestMobs = mobs.slice(0, 20); // Get the closest 20 mobs

    showMobsLocationMenu(player, closestMobs);
}

// Show located mobs in a menu
function showMobsLocationMenu(player, mobs) {
    const mobsLocationForm = new ActionFormData().title("Located Mobs");

    if (mobs.length === 0) {
        mobsLocationForm.button("No mobs found with the specified name", "textures/blocks/barrier");
    } else {
        mobs.forEach(mob => {
            mobsLocationForm.button(`${mob.name} (${mob.distance} blocks): X=${mob.location.x.toFixed(2)}, Y=${mob.location.y.toFixed(2)}, Z=${mob.location.z.toFixed(2)}`);
        });
    }

    mobsLocationForm.show(player).then(response => {
        if (response.canceled) return;

        const selectedMob = mobs[response.selection];
        if (selectedMob) {
            const coords = `X=${selectedMob.location.x.toFixed(2)}, Y=${selectedMob.location.y.toFixed(2)}, Z=${selectedMob.location.z.toFixed(2)}`;
            player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"Cords of the selected mob are ${coords}"}]}`);
        }
    });
}
function showSetCameraForm(player) {
    const setCameraForm = new ModalFormData()
        .title("Set Camera")
        .textField("Enter X coordinate:", "~")
        .textField("Enter Y coordinate:", "~")
        .textField("Enter Z coordinate:", "~");

    setCameraForm.show(player).then(response => {
        if (response.canceled) return;

        const x = response.formValues[0] === "" ? "~" : response.formValues[0];
        const y = response.formValues[1] === "" ? "~" : response.formValues[1];
        const z = response.formValues[2] === "" ? "~" : response.formValues[2];
        const preset = "minecraft:free";

        // Execute camera command in the current dimension
        player.runCommandAsync(`camera @s set ${preset} pos ${x} ${y} ${z}`);
        player.runCommandAsync(`give @s regentstew:exit_camera 1`);

        // Store initial camera data for rotation tracking
        playerCameraData.set(player.name, {
            x: x,
            y: y,
            z: z,
            dimension: player.dimension.id, // Use player's current dimension
            preset: preset
        });
    });
}
function exitCamera(player) {
    const originalData = playerCameraData.get(player.name);
    if (originalData) {
        // Delay the camera clear command by 1 second (20 ticks)
        system.runTimeout(() => {
            player.runCommandAsync(`camera @s clear`)
                .then(() => {
                    // Ensure the camera data is removed from the map
                    playerCameraData.delete(player.name);

                    // Remove the exit camera item from the player's inventory
                    player.runCommandAsync(`clear ${player.name} regentstew:exit_camera`);

                    // Execute the commands a second time to ensure it takes effect
                    system.runTimeout(() => {
                        player.runCommandAsync(`camera @s clear`);
                        player.runCommandAsync(`clear ${player.name} regentstew:exit_camera`);
                    }, 5); // Additional small delay to ensure the commands take effect
                })
                .catch(error => {
                });
        }, 20); // 20 ticks = 1 second
    }
}
// Listen for item use to exit camera
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const itemStack = event.itemStack;

    if (itemStack.typeId === "regentstew:exit_camera") {
        player.runCommandAsync("camera @s clear");
        player.runCommandAsync(`clear @s regentstew:exit_camera`);
    }
});

// Listen for item use to exit camera mode
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const itemStack = event.itemStack;

    if (itemStack.typeId === "regentstew:exit_camera") {
        exitCamera(player);
    }
});
// Store camera data
let playerCameraData = new Map();

// Update camera rotation every tick
system.runInterval(() => {
    world.getPlayers().forEach(player => {
        const cameraData = playerCameraData.get(player.name);
        if (cameraData) {
            const viewDirection = player.getViewDirection();
            const lookX = cameraData.x + viewDirection.x * 3;
            const lookY = cameraData.y + viewDirection.y * 3;
            const lookZ = cameraData.z + viewDirection.z * 3;
            player.runCommandAsync(`camera @s set ${cameraData.preset} pos ${cameraData.x} ${cameraData.y} ${cameraData.z} facing ${lookX} ${lookY} ${lookZ}`);
        }
    });
}, 1); // Update every tick
// Listen for item use to exit camera mode
world.afterEvents.itemUse.subscribe(event => {
    const player = event.source;
    const itemStack = event.itemStack;

    if (itemStack.typeId === "regentstew:exit_camera") {
        exitCamera(player);
    }
});
function showHelpMenu(player) {
    const helpMessages = [
        "§l§1Space Stone§r§b: The Space Stone allows you to teleport to specified coordinates. Use the form to enter X, Y, and Z coordinates, and select the dimension to teleport to.",
        "§l§2Time Stone§r§a: The Time Stone allows you to set the world time, manage time points, and freeze entities in a radius. Use the options to set time, save or go to a time point, or freeze entities.",
        "§l§uPower Stone§r§5: The Power Stone grants temporary buffs and applies Wither effect to nearby entities. Use it to gain strength, speed, resistance, health boost, and regeneration.",
        "§l§6Mind Stone§r§e: The Mind Stone allows you to control mobs or blind them. Use the control mob option to take over a mob, or blind mobs to temporarily disable them. §4TAKE OFF YOUR ARMOR WHEN CONTROLING MOBS OR PEOPLE WILL BE ABLE TO SEE IT! SAME WITH HOLDING ITEMS!",
        "§l§gSoul Stone§r§6: The Soul Stone allows you to damage entities, collect souls, destroy blocks, or summon allies at the cost of souls. Use the options to perform the desired action. Get souls from using the collect souls option in the stone.",
        "§l§4Reality Stone§r§c: The Reality Stone allows you to locate mobs or set the camera. Use the locate mobs option to find specific entities from name, or set your camera to view from a different perspective.",
        "§lGeneral Information§r: The Infinity Gauntlet allows you to use powerful abilities with the stones. The Snap feature, available to admins, kills half of all entities in the dimension. Use the admin tag to access the Snap. You can right click induvidial stones to open their menus alone. To weild any of the stones you must have a full set of infinity armor."
    ];

    helpMessages.forEach(message => {
        player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"${message}"}]}`);
    });
}
function snap(player) {
    const dimension = player.dimension;
    const entities = dimension.getEntities().filter(entity => entity.id !== player.id);
    const halfCount = Math.ceil(entities.length / 2);

    // Shuffle the array of entities
    for (let i = entities.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        [entities[i], entities[j]] = [entities[j], entities[i]];
    }

    // Eliminate half of the entities
    for (let i = 0; i < halfCount; i++) {
        entities[i].kill();
    }

    player.runCommandAsync(`tellraw ${player.name} {"rawtext":[{"text":"§cYou have activated the Snap. Half of all living entities have been wiped out."}]}`);
}
